<?php
namespace Xoogu\CountryFromIP;
class CountryFromIP implements ICountryFromIP
{
	
	protected $_sGeoFunc;
	protected $_sCountryCode;
	protected $_ip2locationType;
	protected $_sIP;
	
	
	/**
	 *Constructs an instance of this class - cannot be called directly, call CountryFromIP::getInstance() instead
	 */
	public function __construct()
	{
		//load settings - use individual blog setting if available, otherwise look for network setting
		$sGeoFunc = get_option('countryFromIp-geoFunc');
		$this->setGeoIPMethod($sGeoFunc ? $sGeoFunc : get_site_option('countryFromIp-geoFunc'));
	}
	
	
	/**
	 *Sets the GeoIP method to use
	 *@param string $sGeoFunc The GeoIP method to use
	 */
	public function setGeoIPMethod($sGeoFunc)
	{
		if ($sGeoFunc) {
			if (strpos($sGeoFunc, 'ip2location') === 0) {
				$this->_ip2locationType = substr($sGeoFunc, 12);
				$sGeoFunc = 'ip2location';
			}
			$this->_sGeoFunc = $sGeoFunc;
		}
	}
	
	
	/**
	 *If Real IP / Forwarded for headers are sent by a proxy, then use that value as the IP address, otherwise use the actual IP address
	 *Note that the headers are not trustworthy
	 *@return string The user's likely IP address
	 */
	public function getIP()
	{
		if ($this->_sIP) {
			return $this->_sIP;
		}
		$sIP = null;
		do {
			if (isset($_SERVER['HTTP_X_REAL_IP'])) {
				$sIP = $this->_sIP = $_SERVER['HTTP_X_REAL_IP'];
				break;
			}
			//comma separated list of ips forwarded for, with first entry being client ip: https://en.wikipedia.org/wiki/X-Forwarded-For#Format
			if (isset($_SERVER['HTTP_X_FORWARDED_FOR'])) {
				$aIPs = explode(',', $_SERVER['HTTP_X_FORWARDED_FOR']);
				//first entry should be the client IP
				$sIP = trim($aIPs[0]);
				if ($sIP) {
					break;
				}
			}
			//There can be multiple Forwarded headers, but PHP will concatenate them into a single comma separated value. We always want the first one at the start of the string
			if (isset($_SERVER['HTTP_FORWARDED'])) {
				//IPv4
				if (preg_match('/^for=([0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3})/i', $_SERVER['HTTP_FORWARDED'], $aMatches)) {
					$sIP = $aMatches[1];
					break;
				//IPv6
				} elseif (preg_match('/^for="\[([^\]]+)\]/i', $_SERVER['HTTP_FORWARDED'], $aMatches)) {
					$sIP = $aMatches[1];
					break;
				}
			}
			$sIP = $_SERVER['REMOTE_ADDR'];
		} while (false);
		if ( ($sIP = filter_var($sIP, FILTER_VALIDATE_IP)) ) {
			return $this->_sIP = $sIP;
		}
	}

	
	/**
	 *Gets the user's country code based on their IP address
	 *@param string $sSpecificIP (optional) The IP to lookup the countrycode for. Providing a specific IP prevents the countrycode from being cached.
	 *@return string The user's country code or an empty string if it couldn't be found
	 */
	public function getCountryFromIP($sSpecificIP=null)
	{
		//Check we have a sGeoFunc set
        if (empty($this->_sGeoFunc)) {
            return '';
        }
		
		//Don't get country code from cache if a specific IP address is given 
		if (empty($sSpecificIP)) {
			//get country from cache if available
			if (!empty($this->_sCountryCode)) {
				return $this->_sCountryCode;
			}
			$sIP = $this->getIP();
		} else {
			$sIP = $sSpecificIP;
		}
		
		if (!$sIP) {
			return '';
		}
		
		$sCountryCode = '';
        switch($this->_sGeoFunc){
            case 'server' :
				//we have to check the value is set since if the server's geoIP can't determine a country code, the variable won't exist rather than being null. And then an undefined index error would be triggered. In PHP7 you can use ?? to coalesce without an undefined index error, but we need to support older PHP versions so use empty.
                $sCountryCode = !empty($_SERVER['GEOIP_COUNTRY_CODE']) ? $_SERVER['GEOIP_COUNTRY_CODE'] : '';
                break;
            case 'php' :
                //PHP GeoIP extension - http://www.php.net/manual/en/function.geoip-country-code-by-name.php
                if(function_exists('geoip_country_code_by_name')){
                    $sCountryCode = geoip_country_code_by_name($sIP);
                }
                break;
			case 'geoip-detect-geolite-city' :
				if (!class_exists('\MaxMind\Db\Reader\Reader')) {
					$geoDir = ABSPATH.'wp-content/plugins/geoip-detect/vendor/maxmind-db/reader/src/MaxMind/Db';
					if (!is_dir($geoDir)) {
						break;
					}
					require_once($geoDir.'/Reader/Decoder.php');
					require_once($geoDir.'/Reader/InvalidDatabaseException.php');
					require_once($geoDir.'/Reader/Metadata.php');
					require_once($geoDir.'/Reader/Util.php');
					require_once($geoDir.'/Reader.php');
				}
				$data_file = ABSPATH.'wp-content/uploads/GeoLite2-City.mmdb';
				if (file_exists($data_file) && $reader = new \MaxMind\Db\Reader($data_file)) {
					try {
						if ( ($record = $reader->get($sIP)) ) {
							$sCountryCode = $record['country']['iso_code'];
						}
					} catch (\Exception $e) {
					}
					$reader->close();
				}
				break;
			case 'ip2location' :
				if (!class_exists('IP2Location')) {
					//tags and variables
					if (file_exists(ABSPATH.'wp-content/plugins/ip2location-'.$this->_ip2locationType.'/ip2location.class.php')) {
						require_once(ABSPATH.'wp-content/plugins/ip2location-'.$this->_ip2locationType.'/ip2location.class.php');
					//redirection and country-blocker
					} elseif (file_exists(ABSPATH.'wp-content/plugins/ip2location-'.$this->_ip2locationType.'/class.IP2Location.php')) {
						require_once(ABSPATH.'wp-content/plugins/ip2location-'.$this->_ip2locationType.'/class.IP2Location.php');
					}
				}
				$ip2location = new IP2Location(ABSPATH.'wp-content/plugins/ip2location-'.$this->_ip2locationType.'/'.get_option( 'ip2location_'.str_replace('-', '_', $this->_ip2locationType).'_database' ));
				$sCountryCode = $ip2location->lookup($sIP,IP2location::COUNTRY_CODE);
				break;
			case 'geoip-detect2' :
				//GeoIP Detection plugin - http://wordpress.org/plugins/geoip-detect/
				//Always use specific IP as geoip_detect2_get_info_from_current_ip only uses 
				if (function_exists('geoip_detect2_get_info_from_ip')) {
					if ( ($userInfo = geoip_detect2_get_info_from_ip($sIP)) ) {
						$sCountryCode = $userInfo->country->isoCode;
					}
				}
				break;
            case 'ip2nation' :
                //Try ip2nation table - http://wordpress.org/plugins/wp-ip2nation-installer/
                if($GLOBALS['wpdb'] && $GLOBALS['wpdb']->get_var("SHOW TABLES LIKE 'ip2nation'")) {
                    //ip2nation uses code of 'UK' for GB in main table. In ip2nationCountries the iso_code_2 is correct (GB), but Ukraine is given as UK (should be UA)
                    $sql = $GLOBALS['wpdb']->prepare('SELECT country FROM ip2nation WHERE ip < INET_ATON(%s) ORDER BY ip DESC LIMIT 0,1', $sIP);
                    $sCountryCode = $GLOBALS['wpdb']->get_var($sql);
					$aFixCountryCode = array('cs' => 'RS', 'yu' => 'RS', 'uk' => 'GB', 'an' => 'CW', 'nt' => null, 'eu' => null, 'ap' => null, '1' => null);
					if (isset($aFixCountryCode[$sCountryCode])) {
						$sCountryCode = $aFixCountryCode[$sCountryCode];
					}
                }
                break;
        }
		$sCountryCode = strtoupper($sCountryCode);
		$sCountryCode = apply_filters('countryCode', $sCountryCode);
		
		//cache the country only for the user's IP
		if (empty($sSpecificIP)) {
			$this->_sCountryCode = $sCountryCode;
		}
		return $sCountryCode;
	}
}